# -*- coding: utf-8 -*-
# @Time    : 8/23/2022 3:41 PM
# @Author  : Jorge Lara
# @Email   : jlara@iee.unsj.edu.ar
# @File    : lib_py_base.py
# @Software: PyCharm

class DSS_xlsx_save:

    def __init__(self, DSS_path: str = None, xlsx_path: str = None, path_save: str = None):
        self.DSS_path = DSS_path
        self.xlsx_path = xlsx_path
        self.path_save = path_save


